@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2003-2006 by Juliusz Chroboczek
+Copyright (c) 2003-2010 by Juliusz Chroboczek
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -38,7 +38,7 @@ DiskCacheEntryPtr diskEntries = NULL, diskEntriesLast = NULL;
 int numDiskEntries = 0;
 int diskCacheDirectoryPermissions = 0700;
 int diskCacheFilePermissions = 0600;
-int diskCacheWriteoutOnClose = (32 * 1024);
+int diskCacheWriteoutOnClose = (64 * 1024);
 
 int maxDiskCacheEntrySize = -1;
 
@@ -49,7 +49,7 @@ int preciseExpiry = 0;
 
 static DiskCacheEntryRec negativeEntry = {
     NULL, NULL,
-    -1, -1, -1, -1, 0, 0, 0, NULL, NULL
+    -1, -1, -1, -1, 0, 0, NULL, NULL
 };
 
 #ifndef LOCAL_ROOT
@@ -128,9 +128,17 @@ checkRoot(AtomPtr root)
     if(!root || root->length == 0)
         return 0;
 
+#ifdef WIN32  /* Require "x:/" or "x:\\" */
+    rc = isalpha(root->string[0]) && (root->string[1] == ':') &&
+         ((root->string[2] == '/') || (root->string[2] == '\\'));
+    if(!rc) {
+        return -2;
+    }
+#else
     if(root->string[0] != '/') {
         return -2;
     }
+#endif
 
     rc = stat(root->string, &ss);
     if(rc < 0)
@@ -290,8 +298,7 @@ localFilename(char *buf, int n, char *key, int len)
 
     if(urlIsSpecial(key, len)) return -1;
 
-    if(localDocumentRoot == NULL ||
-       localDocumentRoot->length <= 0 || localDocumentRoot->string[0] != '/')
+    if(checkRoot(localDocumentRoot) <= 0)
         return -1;
 
     if(n <= localDocumentRoot->length)
@@ -358,11 +365,10 @@ urlDirname(char *buf, int n, const char *url, int len)
     int i, j;
     if(len < 8)
         return -1;
-    if(memcmp(url, "http://", 7) != 0)
+    if(lwrcmp(url, "http://", 7) != 0)
         return -1;
 
-    if(diskCacheRoot == NULL ||
-       diskCacheRoot->length <= 0 || diskCacheRoot->string[0] != '/')
+    if(checkRoot(localDocumentRoot) <= 0)
         return -1;
 
     if(n <= diskCacheRoot->length)
@@ -563,8 +569,7 @@ static int
 writeHeaders(int fd, int *body_offset_return,
              ObjectPtr object, char *chunk, int chunk_len)
 {
-    int n;
-    int rc;
+    int n, rc, error = -1;
     int body_offset = *body_offset_return;
     char *buf = NULL;
     int buf_is_chunk = 0;
@@ -630,8 +635,10 @@ writeHeaders(int fd, int *body_offset_return,
 
     if(body_offset < 0)
         body_offset = n;
-    if(n > body_offset)
+    if(n > body_offset) {
+        error = -2;
         goto fail;
+    }
 
     if(n < body_offset)
         memset(buf + n, 0, body_offset - n);
@@ -690,7 +697,7 @@ writeHeaders(int fd, int *body_offset_return,
         dispose_chunk(buf);
     else
         free(buf);
-    return -1;
+    return error;
 }
 
 typedef struct _MimeEntry {
@@ -947,7 +954,7 @@ validateEntry(ObjectPtr object, int fd,
 
     if(!location || strlen(location) != object->key_size ||
        memcmp(location, object->key, object->key_size) != 0) {
-        do_log(L_ERROR, "Inconsistent cache file for %s.\n", location);
+        do_log(L_ERROR, "Inconsistent cache file for %s.\n", scrub(location));
         goto invalid;
     }
 
@@ -955,7 +962,7 @@ validateEntry(ObjectPtr object, int fd,
         polipo_age = date;
 
     if(polipo_age < 0) {
-        do_log(L_ERROR, "Undated disk entry for %s.\n", location);
+        do_log(L_ERROR, "Undated disk entry for %s.\n", scrub(location));
         goto invalid;
     }
 
@@ -1035,6 +1042,8 @@ validateEntry(ObjectPtr object, int fd,
         dirty = 1;
 
     object->cache_control |= cache_control.flags;
+    object->max_age = cache_control.max_age;
+    object->s_maxage = cache_control.s_maxage;
 
     if(object->age < 0) object->age = object->date;
     if(object->age < 0) object->age = 0; /* a long time ago */
@@ -1124,19 +1133,13 @@ revalidateDiskEntry(ObjectPtr object)
     return 1;
 }
 
-static inline int
-objectHasDiskEntry(ObjectPtr object)
-{
-    return object->disk_entry && object->disk_entry != &negativeEntry;
-}
-
 static DiskCacheEntryPtr
-makeDiskEntry(ObjectPtr object, int writeable, int create)
+makeDiskEntry(ObjectPtr object, int create)
 {
     DiskCacheEntryPtr entry = NULL;
     char buf[1024];
     int fd = -1;
-    int negative = 0, isWriteable = 0, size = -1, name_len = -1;
+    int negative = 0, size = -1, name_len = -1;
     char *name = NULL;
     off_t offset = -1;
     int body_offset = -1;
@@ -1144,8 +1147,8 @@ makeDiskEntry(ObjectPtr object, int writeable, int create)
     int local = (object->flags & OBJECT_LOCAL) != 0;
     int dirty = 0;
 
-    if(local && (writeable || create))
-        return NULL;
+   if(local && create)
+       return NULL;
 
     if(!local && !(object->flags & OBJECT_PUBLIC))
         return NULL;
@@ -1163,7 +1166,7 @@ makeDiskEntry(ObjectPtr object, int writeable, int create)
     if(object->disk_entry) {
         entry = object->disk_entry;
         CHECK_ENTRY(entry);
-        if(entry != &negativeEntry && (!writeable || entry->writeable)) {
+        if(entry != &negativeEntry) {
             /* We'll keep the entry -- put it at the front. */
             if(entry != diskEntries && entry != &negativeEntry) {
                 entry->previous->next = entry->next;
@@ -1196,14 +1199,8 @@ makeDiskEntry(ObjectPtr object, int writeable, int create)
             return NULL;
         name_len = urlFilename(buf, 1024, object->key, object->key_size);
         if(name_len < 0) return NULL;
-        if(!negative) {
-            isWriteable = 1;
+        if(!negative)
             fd = open(buf, O_RDWR | O_BINARY);
-            if(fd < 0 && !writeable && errno == EACCES) {
-                writeable = 0;
-                fd = open(buf, O_RDONLY | O_BINARY);
-            }
-        }
         if(fd >= 0) {
             rc = validateEntry(object, fd, &body_offset, &offset);
             if(rc >= 0) {
@@ -1215,7 +1212,7 @@ makeDiskEntry(ObjectPtr object, int writeable, int create)
                 if(rc < 0 && errno != ENOENT) {
                     do_log_error(L_WARN,  errno,
                                  "Couldn't unlink stale disk entry %s", 
-                                 buf);
+                                 scrub(buf));
                     /* But continue -- it's okay to have stale entries. */
                 }
             }
@@ -1223,7 +1220,6 @@ makeDiskEntry(ObjectPtr object, int writeable, int create)
 
         if(fd < 0 && create && name_len > 0 && 
            !(object->flags & OBJECT_INITIAL)) {
-            isWriteable = 1;
             fd = createFile(buf, diskCacheRoot->length);
             if(fd < 0)
                 return NULL;
@@ -1242,7 +1238,7 @@ makeDiskEntry(ObjectPtr object, int writeable, int create)
                     if(rc < 0 && errno != ENOENT)
                         do_log_error(L_ERROR, errno,
                                      "Couldn't unlink truncated entry %s", 
-                                     buf);
+                                     scrub(buf));
                     close(fd);
                     return NULL;
                 }
@@ -1261,7 +1257,6 @@ makeDiskEntry(ObjectPtr object, int writeable, int create)
             localFilename(buf, 1024, object->key, object->key_size);
         if(name_len < 0)
             return NULL;
-        isWriteable = 0;
         fd = open(buf, O_RDONLY | O_BINARY);
         if(fd >= 0) {
             if(validateEntry(object, fd, &body_offset, NULL) < 0) {
@@ -1302,7 +1297,6 @@ makeDiskEntry(ObjectPtr object, int writeable, int create)
     entry->offset = offset;
     entry->size = size;
     entry->metadataDirty = dirty;
-    entry->writeable = isWriteable;
 
     entry->next = diskEntries;
     if(diskEntries)
@@ -1341,7 +1335,7 @@ rewriteEntry(ObjectPtr object)
         close(fd);
         return -1;
     }
-    entry = makeDiskEntry(object, 1, 1);
+    entry = makeDiskEntry(object, 1);
     if(!entry) {
         close(fd);
         return -1;
@@ -1374,15 +1368,20 @@ rewriteEntry(ObjectPtr object)
     while(1) {
         CHECK_ENTRY(entry);
         n = read(fd, buf, bufsize);
+        if(n < 0 && errno == EINTR)
+            continue;
         if(n <= 0)
             goto done;
         rc = entrySeek(entry, entry->body_offset + offset);
         if(rc < 0)
             goto done;
+    write_again:
         rc = write(entry->fd, buf, n);
         if(rc >= 0) {
             entry->offset += rc;
             entry->size += rc;
+        } else if(errno == EINTR) {
+            goto write_again;
         }
         if(rc < n)
             goto done;
@@ -1409,7 +1408,7 @@ destroyDiskEntry(ObjectPtr object, int d)
     assert(!entry || !entry->local || !d);
 
     if(d && !entry)
-        entry = makeDiskEntry(object, 1, 0);
+        entry = makeDiskEntry(object, 0);
 
     CHECK_ENTRY(entry);
 
@@ -1430,18 +1429,22 @@ destroyDiskEntry(ObjectPtr object, int d)
             urc = unlink(entry->filename);
             if(urc < 0)
                 do_log_error(L_WARN, errno, 
-                             "Couldn't unlink %s", entry->filename);
+                             "Couldn't unlink %s", scrub(entry->filename));
         }
     } else {
         if(entry && entry->metadataDirty)
             writeoutMetadata(object);
-        makeDiskEntry(object, 1, 0);
+        makeDiskEntry(object, 0);
         /* rewriteDiskEntry may change the disk entry */
         entry = object->disk_entry;
         if(entry == NULL || entry == &negativeEntry)
             return 0;
-        if(entry->writeable && diskCacheWriteoutOnClose > 0)
+        if(diskCacheWriteoutOnClose > 0) {
             reallyWriteoutToDisk(object, -1, diskCacheWriteoutOnClose);
+            entry = object->disk_entry;
+            if(entry == NULL || entry == &negativeEntry)
+                return 0;
+        }
     }
  again:
     rc = close(entry->fd);
@@ -1477,7 +1480,7 @@ destroyDiskEntry(ObjectPtr object, int d)
 ObjectPtr 
 objectGetFromDisk(ObjectPtr object)
 {
-    DiskCacheEntryPtr entry = makeDiskEntry(object, 0, 0);
+    DiskCacheEntryPtr entry = makeDiskEntry(object, 0);
     if(!entry) return NULL;
     return object;
 }
@@ -1529,7 +1532,7 @@ objectFillFromDisk(ObjectPtr object, int offset, int chunks)
 
     /* This has the side-effect of revalidating the entry, which is
        what makes HEAD requests work. */
-    entry = makeDiskEntry(object, 0, 0);
+    entry = makeDiskEntry(object, 0);
     if(!entry)
         return 0;
                 
@@ -1660,7 +1663,7 @@ reallyWriteoutToDisk(ObjectPtr object, int upto, int max)
     if((object->flags & OBJECT_DISK_ENTRY_COMPLETE) && !object->disk_entry)
         return 0;
 
-    entry = makeDiskEntry(object, 1, 1);
+    entry = makeDiskEntry(object, 1);
     if(!entry) return 0;
 
     assert(!entry->local);
@@ -1680,25 +1683,14 @@ reallyWriteoutToDisk(ObjectPtr object, int upto, int max)
     if(entry->size >= upto)
         goto done;
 
-    if(!entry->writeable) {
-        entry = makeDiskEntry(object, 1, 1);
-        if(!entry)
-            return 0;
-        if(!entry->writeable)
-            return 0;
-        diskEntrySize(object);
-        if(entry->size < 0)
-            return 0;
-    }
-
     offset = entry->size;
 
     /* Avoid a seek in case we start writing at the beginning */
     if(offset == 0 && entry->metadataDirty) {
         writeoutMetadata(object);
         /* rewriteDiskEntry may change the entry */
-        entry = makeDiskEntry(object, 1, 0);
-        if(entry == NULL || !entry->writeable)
+        entry = makeDiskEntry(object, 0);
+        if(entry == NULL)
             return 0;
     }
 
@@ -1750,7 +1742,7 @@ writeoutMetadata(ObjectPtr object)
        (object->flags & OBJECT_LOCAL))
         return 0;
     
-    entry = makeDiskEntry(object, 1, 0);
+    entry = makeDiskEntry(object, 0);
     if(entry == NULL || entry == &negativeEntry)
         goto fail;
 
@@ -1818,7 +1810,7 @@ readDiskObject(char *filename, struct stat *sb)
     if(sb == NULL) {
         rc = stat(filename, &ss);
         if(rc < 0) {
-            do_log_error(L_WARN, errno, "Couldn't stat %s", filename);
+            do_log_error(L_WARN, errno, "Couldn't stat %s", scrub(filename));
             return NULL;
         }
         sb = &ss;
@@ -1891,6 +1883,7 @@ readDiskObject(char *filename, struct stat *sb)
         atime = -1;
         date = -1;
         last_modified = -1;
+        expires = -1;
     } else {
         goto fail;
     }
@@ -1941,7 +1934,7 @@ processObject(DiskObjectPtr dobjects, char *filename, struct stat *sb)
 
     dobject = readDiskObject((char*)filename, sb);
     if(dobject == NULL)
-        return 0;
+        return dobjects;
 
     if(!dobjects ||
        (c = strcmp(dobject->location, dobjects->location)) <= 0) {
@@ -2371,11 +2364,12 @@ expireFile(char *filename, struct stat *sb,
 
     dobject = readDiskObject(filename, sb);
     if(!dobject) {
-        do_log(L_ERROR, "Incorrect disk entry %s -- removing.\n", filename);
+        do_log(L_ERROR, "Incorrect disk entry %s -- removing.\n",
+               scrub(filename));
         rc = unlink(filename);
         if(rc < 0) {
             do_log_error(L_ERROR, errno,
-                         "Couldn't unlink %s", filename);
+                         "Couldn't unlink %s", scrub(filename));
             return ret;
         } else {
             (*unlinked)++;
@@ -2390,12 +2384,13 @@ expireFile(char *filename, struct stat *sb,
     if(t > current_time.tv_sec)
         do_log(L_WARN, 
                "Disk entry %s (%s) has access time in the future.\n",
-               dobject->location, dobject->filename);
+               scrub(dobject->location), scrub(dobject->filename));
     
     if(t < current_time.tv_sec - diskCacheUnlinkTime) {
         rc = unlink(dobject->filename);
         if(rc < 0) {
-            do_log_error(L_ERROR, errno, "Couldn't unlink %s", filename);
+            do_log_error(L_ERROR, errno, "Couldn't unlink %s",
+                         scrub(filename));
         } else {
             (*unlinked)++;
             ret = 0;
@@ -2408,7 +2403,8 @@ expireFile(char *filename, struct stat *sb,
         fd = open(dobject->filename, O_RDONLY | O_BINARY, 0);
         rc = unlink(dobject->filename);
         if(rc < 0) {
-            do_log_error(L_ERROR, errno, "Couldn't unlink %s", filename);
+            do_log_error(L_ERROR, errno, "Couldn't unlink %s",
+                         scrub(filename));
             close(fd);
             fd = -1;
         } else {
@@ -2469,11 +2465,11 @@ expireDiskObjects()
                 else if(errno != ENOTEMPTY && errno != EEXIST)
                     do_log_error(L_ERROR, errno,
                                  "Couldn't remove directory %s",
-                                 fe->fts_accpath);
+                                 scrub(fe->fts_accpath));
                 continue;
             } else if(fe->fts_info == FTS_NS) {
                 do_log_error(L_ERROR, fe->fts_errno, "Couldn't stat file %s",
-                             fe->fts_accpath);
+                             scrub(fe->fts_accpath));
                 continue;
             } else if(fe->fts_info == FTS_ERR) {
                 do_log_error(L_ERROR, fe->fts_errno,
