@@ -62,7 +62,7 @@ static inline void
 timeval_minus(struct timeval *d,
               const struct timeval *s1, const struct timeval *s2)
 {
-    if(s1->tv_usec > s2->tv_usec) {
+    if(s1->tv_usec >= s2->tv_usec) {
         d->tv_usec = s1->tv_usec - s2->tv_usec;
         d->tv_sec = s1->tv_sec - s2->tv_sec;
     } else {
