@@ -229,7 +229,7 @@ do_scheduled_stream(int status, FdEventHandlerPtr event)
     StreamRequestPtr request = (StreamRequestPtr)&event->data;
     int rc, done, i;
     struct iovec iov[6];
-    int chunk_header_len, chunk_trailer_len;
+    int chunk_header_len;
     char chunk_header[10];
     int len12 = request->len + request->len2;
     int len123 = 
@@ -248,10 +248,8 @@ do_scheduled_stream(int status, FdEventHandlerPtr event)
                IO_WRITE);
         if(request->operation & IO_CHUNKED) {
             chunk_header_len = chunkHeaderLen(len123);
-            chunk_trailer_len = 2;
         } else {
             chunk_header_len = 0;
-            chunk_trailer_len = 0;
         }
 
         if(request->offset < -chunk_header_len) {
@@ -649,7 +647,7 @@ schedule_accept(int fd,
     request.fd = fd;
     request.handler = handler;
     request.data = data;
-    event = registerFdEvent(fd, POLLOUT|POLLIN, 
+    event = registerFdEvent(fd, POLLIN, 
                             do_scheduled_accept, sizeof(request), &request);
     if(!event) {
         done = (*handler)(-ENOMEM, NULL, NULL);
@@ -724,8 +722,11 @@ create_listener(char *address, int port,
         return NULL;
     }
 
+#ifndef WIN32
+    /* on WIN32 SO_REUSEADDR allows two sockets bind to the same port */
     rc = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one));
     if(rc < 0) do_log_error(L_WARN, errno, "Couldn't set SO_REUSEADDR");
+#endif
 
     if(inet6) {
 #ifdef HAVE_IPv6
@@ -779,7 +780,7 @@ create_listener(char *address, int port,
         return NULL;
     }
         
-    rc = listen(fd, 32);
+    rc = listen(fd, 1024);
     if(rc < 0) {
         do_log_error(L_ERROR, errno, "Couldn't listen");
         CLOSE(fd);
@@ -801,8 +802,8 @@ create_listener(char *address, int port,
 int
 setNonblocking(int fd, int nonblocking)
 {
-#ifdef MINGW
-    return mingw_setnonblocking(fd, nonblocking);
+#ifdef WIN32 /*MINGW*/
+    return win32_setnonblocking(fd, nonblocking);
 #else
     int rc;
     rc = fcntl(fd, F_GETFL, 0);
