@@ -55,7 +55,7 @@ preinitLocal()
 
 static void fillSpecialObject(ObjectPtr, void (*)(FILE*, char*), void*);
 
-int 
+int
 httpLocalRequest(ObjectPtr object, int method, int from, int to,
                  HTTPRequestPtr requestor, void *closure)
 {
@@ -67,14 +67,10 @@ httpLocalRequest(ObjectPtr object, int method, int from, int to,
                                   requestor, closure);
 
     if(method >= METHOD_POST) {
-        httpClientError(requestor, 405, internAtom("Method not allowed"));
-        requestor->connection->flags &= ~CONN_READER;
-        return 1;
-    }
-
-    /* objectFillFromDisk already did the real work but we have to
-       make sure we don't get into an infinite loop. */
-    if(object->flags & OBJECT_INITIAL) {
+        abortObject(object, 405, internAtom("Method not allowed"));
+    } else if(object->flags & OBJECT_INITIAL) {
+        /* objectFillFromDisk already did the real work but we have to
+           make sure we don't get into an infinite loop. */
         abortObject(object, 404, internAtom("Not found"));
     }
     object->age = current_time.tv_sec;
@@ -309,6 +305,12 @@ httpSpecialSideRequest(ObjectPtr object, int method, int from, int to,
         return 1;
     }
 
+    if(requestor->flags & REQUEST_WAIT_CONTINUE) {
+        httpClientError(requestor, 417, internAtom("Expectation failed"));
+        requestor->connection->flags &= ~CONN_READER;
+        return 1;
+    }
+
     return httpSpecialDoSide(requestor);
 }
 
@@ -565,7 +567,7 @@ fillSpecialObject(ObjectPtr object, void (*fn)(FILE*, char*), void* closure)
             abortObject(object, 503,
                         internAtom("Couldn't allocate request\n"));
             notifyObject(object);
-            /* specialRequestHandler will take care of the rest. */
+            return;
         } else {
             request->buf = get_chunk();
             if(request->buf == NULL) {
@@ -575,6 +577,7 @@ fillSpecialObject(ObjectPtr object, void (*fn)(FILE*, char*), void* closure)
                 abortObject(object, 503,
                             internAtom("Couldn't allocate request\n"));
                 notifyObject(object);
+                return;
             }
         }
         object->flags |= OBJECT_INPROGRESS;
