@@ -75,6 +75,13 @@ circularBufferEmpty(CircularBufferPtr buf)
      return buf->head == buf->tail;
 }
 
+static void
+logTunnel(TunnelPtr tunnel, int blocked)
+{
+    do_log(L_TUNNEL,"tunnel %s:%d %s\n", tunnel->hostname->string, tunnel->port,
+	   blocked ? "blocked" : "allowed");
+}
+
 static TunnelPtr
 makeTunnel(int fd, char *buf, int offset, int len)
 {
@@ -90,12 +97,17 @@ makeTunnel(int fd, char *buf, int offset, int len)
     tunnel->flags = 0;
     tunnel->fd1 = fd;
     tunnel->fd2 = -1;
-    tunnel->buf1.buf = NULL;
-    tunnel->buf1.tail = 0;
-    tunnel->buf1.head = 0;
-    tunnel->buf2.buf = buf;
+    tunnel->buf1.buf = buf;
+    if(offset == len) {
+        tunnel->buf1.tail = 0;
+        tunnel->buf1.head = 0;
+    } else {
+        tunnel->buf1.tail = offset;
+        tunnel->buf1.head = len;
+    }
+    tunnel->buf2.buf = NULL;
     tunnel->buf2.tail = 0;
-    tunnel->buf2.head = offset;
+    tunnel->buf2.head = 0;
     return tunnel;
 }
 
@@ -127,6 +139,14 @@ do_tunnel(int fd, char *buf, int offset, int len, AtomPtr url)
         return;
     }
 
+    if(proxyOffline) {
+        do_log(L_INFO, "Attemted CONNECT when disconnected.\n");
+        releaseAtom(url);
+        tunnelError(tunnel, 502,
+                    internAtom("Cannot CONNECT when disconnected."));
+        return;
+    }
+
     p = memrchr(url->string, ':', url->length);
     q = NULL;
     if(p)
@@ -151,6 +171,16 @@ do_tunnel(int fd, char *buf, int offset, int len, AtomPtr url)
     }
     tunnel->port = port;
     
+    if (tunnelIsMatched(url->string, url->length, 
+			tunnel->hostname->string, tunnel->hostname->length)) {
+        releaseAtom(url);
+        tunnelError(tunnel, 404, internAtom("Forbidden tunnel"));
+	logTunnel(tunnel,1);
+        return;
+    }
+    
+    logTunnel(tunnel,0);
+    
     releaseAtom(url);
 
     if(socksParentProxy)
@@ -230,23 +260,30 @@ tunnelHandlerParent(int fd, TunnelPtr tunnel)
     char *message;
     int n;
 
-    tunnel->fd2 = fd;
-
-    tunnel->buf2.buf = get_chunk();
-    if(tunnel->buf2.buf == NULL) {
+    if(tunnel->buf1.buf == NULL)
+        tunnel->buf1.buf = get_chunk();
+    if(tunnel->buf1.buf == NULL) {
         message = "Couldn't allocate buffer";
         goto fail;
     }
+    if(tunnel->buf1.tail != tunnel->buf1.head) {
+        message = "Pipelined connect to parent proxy not implemented";
+        goto fail;
+    }
 
-    n = snnprintf(tunnel->buf2.buf, 0, CHUNK_SIZE,
-                  "CONNECT %s:%d HTTP/1.1"
-                  "\r\n\r\n",
+    n = snnprintf(tunnel->buf1.buf, tunnel->buf1.tail, CHUNK_SIZE,
+                  "CONNECT %s:%d HTTP/1.1",
                   tunnel->hostname->string, tunnel->port);
+    if (parentAuthCredentials)
+        n = buildServerAuthHeaders(tunnel->buf1.buf, n, CHUNK_SIZE,
+                                   parentAuthCredentials);
+    n = snnprintf(tunnel->buf1.buf, n, CHUNK_SIZE, "\r\n\r\n");
+
     if(n < 0) {
         message = "Buffer overflow";
         goto fail;
     }
-    tunnel->buf2.head = n;
+    tunnel->buf1.head = n;
     tunnelDispatch(tunnel);
     return 1;
 
@@ -261,23 +298,23 @@ static int
 tunnelHandlerCommon(int fd, TunnelPtr tunnel)
 {
     const char *message = "HTTP/1.1 200 Tunnel established\r\n\r\n";
-    assert(tunnel->buf1.buf == NULL);
+
+    tunnel->fd2 = fd;
 
     if(parentHost)
         return tunnelHandlerParent(fd, tunnel);
 
-    tunnel->buf1.buf = get_chunk();
-    if(tunnel->buf1.buf == NULL) {
+    if(tunnel->buf2.buf == NULL)
+        tunnel->buf2.buf = get_chunk();
+    if(tunnel->buf2.buf == NULL) {
         CLOSE(fd);
         tunnelError(tunnel, 501, internAtom("Couldn't allocate buffer"));
         return 1;
     }
 
-    tunnel->fd2 = fd;
-
     memcpy(tunnel->buf2.buf, message, MIN(CHUNK_SIZE - 1, strlen(message)));
     tunnel->buf2.head = MIN(CHUNK_SIZE - 1, strlen(message));
-        
+
     tunnelDispatch(tunnel);
     return 1;
 }
@@ -425,7 +462,7 @@ tunnelRead1Handler(int status,
 {
     TunnelPtr tunnel = request->data;
     if(status) {
-        if(status < 0)
+        if(status < 0 && status != -EPIPE && status != -ECONNRESET)
             do_log_error(L_ERROR, -status, "Couldn't read from client");
         tunnel->flags |= TUNNEL_EOF1;
         goto done;
@@ -446,7 +483,7 @@ tunnelRead2Handler(int status,
 {
     TunnelPtr tunnel = request->data;
     if(status) {
-        if(status < 0)
+        if(status < 0 && status != -EPIPE && status != -ECONNRESET)
             do_log_error(L_ERROR, -status, "Couldn't read from server");
         tunnel->flags |= TUNNEL_EOF2;
         goto done;
